'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var render = require('miniapp-render');
var render__default = _interopDefault(render);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(n);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o) {
  var i = 0;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  i = o[Symbol.iterator]();
  return i.next.bind(i);
}

var Event = render.$$adapter.Event;
function callSingleEvent(eventName, evt, nativeComponent) {
  var domNode = nativeComponent.getDomNodeFromEvt(eventName, evt);
  if (!domNode) return;
  domNode.$$trigger(eventName, {
    event: new Event({
      timeStamp: evt && evt.timeStamp,
      touches: evt && evt.touches,
      changedTouches: evt && evt.changedTouches,
      name: eventName,
      target: domNode,
      eventPhase: Event.AT_TARGET,
      detail: evt && evt.detail,
      $$extra: evt && evt.extra
    }),
    currentTarget: domNode
  });
}

var coverImage = {
  name: 'cover-image',
  props: [{
    name: 'src',
    get: function get(domNode) {
      return domNode.src;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onCoverImageLoad: function onCoverImageLoad(evt) {
      callSingleEvent('load', evt, this);
    },
    onCoverImageError: function onCoverImageError(evt) {
      callSingleEvent('error', evt, this);
    }
  }
};

var coverView = {
  name: 'cover-view',
  props: [{
    name: 'scrollTop',
    get: function get(domNode) {
      var value = domNode.getAttribute('scroll-top');
      return value !== undefined && !isNaN(+value) ? +value : '';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

var movableArea = {
  name: 'movable-area',
  props: [{
    name: 'scaleArea',
    get: function get(domNode) {
      return !!domNode.getAttribute('scale-area');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

// eslint-disable-next-line import/no-extraneous-dependencies
var ScrollView = {
  name: 'scroll-view',
  props: [{
    name: 'scrollX',
    get: function get(domNode) {
      return !!domNode.getAttribute('scroll-x');
    }
  }, {
    name: 'scrollY',
    get: function get(domNode) {
      return !!domNode.getAttribute('scroll-y');
    }
  }, {
    name: 'upperThreshold',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('upper-threshold'), 10);
      return !isNaN(value) ? value : 50;
    }
  }, {
    name: 'lowerThreshold',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('lower-threshold'), 10);
      return !isNaN(value) ? value : 50;
    }
  }, {
    name: 'scrollTop',
    canBeUserChanged: true,
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('scroll-top'), 10);
      return !isNaN(value) ? value : '';
    }
  }, {
    name: 'scrollLeft',
    canBeUserChanged: true,
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('scroll-left'), 10);
      return !isNaN(value) ? value : '';
    }
  }, {
    name: 'scrollIntoView',
    canBeUserChanged: true,
    get: function get(domNode) {
      return domNode.getAttribute('scroll-into-view') || '';
    }
  }, {
    name: 'scrollWithAnimation',
    get: function get(domNode) {
      return !!domNode.getAttribute('scroll-with-animation');
    }
  }, {
    name: 'scrollAnimationDuration',
    get: function get(domNode) {
      return domNode.getAttribute('scroll-animation-duration');
    }
  }, {
    name: 'enableBackToTop',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-back-to-top');
    }
  }, {
    name: 'enableFlex',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-flex');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onScrollViewScrolltoupper: function onScrollViewScrolltoupper(evt) {
      callSingleEvent('scrolltoupper', evt, this);
    },
    onScrollViewScrolltolower: function onScrollViewScrolltolower(evt) {
      callSingleEvent('scrolltolower', evt, this);
    },
    onScrollViewScroll: function onScrollViewScroll(evt) {
      var domNode = this.getDomNodeFromEvt('scroll', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('scroll-into-view', '');
      domNode.$$setAttributeWithoutUpdate('scroll-top', evt.detail.scrollTop);
      domNode.$$setAttributeWithoutUpdate('scroll-left', evt.detail.scrollLeft);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.scrollIntoView = '';
      domNode.__oldValues.scrollTop = evt.detail.scrollTop;
      domNode.__oldValues.scrollLeft = evt.detail.scrollLeft;
      callSingleEvent('scroll', evt, this);
    }
  }
};

{
  Object.assign(ScrollView.handles, {
    onScrollViewRefresherPulling: function onScrollViewRefresherPulling(evt) {
      callSingleEvent('refresherpulling', evt, this);
    },
    onScrollViewRefresherRefresh: function onScrollViewRefresherRefresh(evt) {
      callSingleEvent('refresherrefresh', evt, this);
    },
    onScrollViewRefresherRestore: function onScrollViewRefresherRestore(evt) {
      callSingleEvent('refresherrestore', evt, this);
    },
    onScrollViewRefresherAbort: function onScrollViewRefresherAbort(evt) {
      callSingleEvent('refresherabort', evt, this);
    }
  });
}

var swiper = {
  name: 'swiper',
  props: [{
    name: 'indicatorDots',
    get: function get(domNode) {
      return !!domNode.getAttribute('indicator-dots');
    }
  }, {
    name: 'indicatorColor',
    get: function get(domNode) {
      return domNode.getAttribute('indicator-color') || 'rgba(0, 0, 0, .3)';
    }
  }, {
    name: 'indicatorActiveColor',
    get: function get(domNode) {
      return domNode.getAttribute('indicator-active-color') || '#000000';
    }
  }, {
    name: 'autoplay',
    get: function get(domNode) {
      return !!domNode.getAttribute('autoplay');
    }
  }, {
    name: 'current',
    canBeUserChanged: true,
    get: function get(domNode) {
      return +domNode.getAttribute('current') || 0;
    }
  }, {
    name: 'interval',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('interval'), 10);
      return !isNaN(value) ? value : 5000;
    }
  }, {
    name: 'duration',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('duration'), 10);
      return !isNaN(value) ? value : 500;
    }
  }, {
    name: 'circular',
    get: function get(domNode) {
      return !!domNode.getAttribute('circular');
    }
  }, {
    name: 'vertical',
    get: function get(domNode) {
      return !!domNode.getAttribute('vertical');
    }
  }, {
    name: 'previousMargin',
    get: function get(domNode) {
      return domNode.getAttribute('previous-margin') || '0px';
    }
  }, {
    name: 'nextMargin',
    get: function get(domNode) {
      return domNode.getAttribute('next-margin') || '0px';
    }
  }, {
    name: 'displayMultipleItems',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('display-multiple-items'), 10);
      return !isNaN(value) ? value : 1;
    }
  }, {
    name: 'skipHiddenItemLayout',
    get: function get(domNode) {
      return !!domNode.getAttribute('skip-hidden-item-layout');
    }
  }, {
    name: 'easingFunction',
    get: function get(domNode) {
      return domNode.getAttribute('easing-function') || 'default';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onSwiperChange: function onSwiperChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('current', evt.detail.current);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.current = evt.detail.current;
      callSingleEvent('change', evt, this);
    },
    onSwiperTransition: function onSwiperTransition(evt) {
      callSingleEvent('transition', evt, this);
    },
    onSwiperAnimationfinish: function onSwiperAnimationfinish(evt) {
      callSingleEvent('animationfinish', evt, this);
    }
  }
};

var view = {
  name: 'view',
  props: [{
    name: 'hoverClass',
    get: function get(domNode) {
      return domNode.getAttribute('hover-class') || 'none';
    }
  }, {
    name: 'hoverStopPropagation',
    get: function get(domNode) {
      return !!domNode.getAttribute('hover-stop-propagation');
    }
  }, {
    name: 'hoverStartTime',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('hover-start-time'), 10);
      return !isNaN(value) ? value : 50;
    }
  }, {
    name: 'hoverStayTime',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('hover-stay-time'), 10);
      return !isNaN(value) ? value : 400;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

var icon = {
  name: 'icon',
  props: [{
    name: 'type',
    get: function get(domNode) {
      return domNode.getAttribute('type') || '';
    }
  }, {
    name: 'size',
    get: function get(domNode) {
      return domNode.getAttribute('size') || '23';
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

var progress = {
  name: 'progress',
  props: [{
    name: 'percent',
    get: function get(domNode) {
      return +domNode.getAttribute('percent') || 0;
    }
  }, {
    name: 'showInfo',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-info');
    }
  }, {
    name: 'borderRadius',
    get: function get(domNode) {
      return domNode.getAttribute('border-radius') || '0';
    }
  }, {
    name: 'fontSize',
    get: function get(domNode) {
      return domNode.getAttribute('font-size') || '16';
    }
  }, {
    name: 'strokeWidth',
    get: function get(domNode) {
      return domNode.getAttribute('stroke-width') || '6';
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '#09BB07';
    }
  }, {
    name: 'activeColor',
    get: function get(domNode) {
      return domNode.getAttribute('active-color') || '#09BB07';
    }
  }, {
    name: 'backgroundColor',
    get: function get(domNode) {
      return domNode.getAttribute('background-color') || '#EBEBEB';
    }
  }, {
    name: 'active',
    get: function get(domNode) {
      return !!domNode.getAttribute('active');
    }
  }, {
    name: 'activeMode',
    get: function get(domNode) {
      return domNode.getAttribute('active-mode') || 'backwards';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onProgressActiveEnd: function onProgressActiveEnd(evt) {
      callSingleEvent('activeend', evt, this);
    }
  }
};

var text = {
  name: 'text',
  props: [{
    name: 'selectable',
    get: function get(domNode) {
      return !!domNode.getAttribute('selectable');
    }
  }, {
    name: 'space',
    get: function get(domNode) {
      return domNode.getAttribute('space') || '';
    }
  }, {
    name: 'decode',
    get: function get(domNode) {
      return !!domNode.getAttribute('decode');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

/* global isWeChatMiniProgram */
var props = [{
  name: 'nodes',
  get: function get(domNode) {
    return domNode.getAttribute('nodes') || [];
  }
}];

{
  props.push({
    name: 'space',
    get: function get(domNode) {
      return domNode.getAttribute('space') || '';
    }
  });
}

var richText = {
  name: 'rich-text',
  props: props,
  handles: {}
};

var button = {
  name: 'button',
  props: [{
    name: 'size',
    get: function get(domNode) {
      return domNode.getAttribute('size') || 'default';
    }
  }, {
    name: 'type',
    get: function get(domNode) {
      return domNode.getAttribute('type') || 'default';
    }
  }, {
    name: 'plain',
    get: function get(domNode) {
      return !!domNode.getAttribute('plain');
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return !!domNode.getAttribute('disabled');
    }
  }, {
    name: 'loading',
    get: function get(domNode) {
      return !!domNode.getAttribute('loading');
    }
  }, {
    name: 'formType',
    get: function get(domNode) {
      return domNode.getAttribute('form-type') || '';
    }
  }, {
    name: 'openType',
    get: function get(domNode) {
      return domNode.getAttribute('open-type') || '';
    }
  }, {
    name: 'hoverClass',
    get: function get(domNode) {
      return domNode.getAttribute('hover-class') || 'button-hover';
    }
  }, {
    name: 'hoverStopPropagation',
    get: function get(domNode) {
      return !!domNode.getAttribute('hover-stop-propagation');
    }
  }, {
    name: 'hoverStartTime',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('hover-start-time'), 10);
      return !isNaN(value) ? value : 20;
    }
  }, {
    name: 'hoverStayTime',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('hover-stay-time'), 10);
      return !isNaN(value) ? value : 70;
    }
  }, {
    name: 'lang',
    get: function get(domNode) {
      return domNode.getAttribute('lang') || 'en';
    }
  }, {
    name: 'sessionFrom',
    get: function get(domNode) {
      return domNode.getAttribute('session-from') || '';
    }
  }, {
    name: 'sendMessageTitle',
    get: function get(domNode) {
      return domNode.getAttribute('send-message-title') || '';
    }
  }, {
    name: 'sendMessagePath',
    get: function get(domNode) {
      return domNode.getAttribute('send-message-path') || '';
    }
  }, {
    name: 'sendMessageImg',
    get: function get(domNode) {
      return domNode.getAttribute('send-message-img') || '';
    }
  }, {
    name: 'appParameter',
    get: function get(domNode) {
      return domNode.getAttribute('app-parameter') || '';
    }
  }, {
    name: 'showMessageCard',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-message-card');
    }
  }, {
    name: 'businessId',
    get: function get(domNode) {
      return domNode.getAttribute('business-id') || '';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onButtonGetUserInfo: function onButtonGetUserInfo(evt) {
      callSingleEvent('getuserinfo', evt, this);
    },
    onButtonContact: function onButtonContact(evt) {
      callSingleEvent('contact', evt, this);
    },
    onButtonGetPhoneNumber: function onButtonGetPhoneNumber(evt) {
      callSingleEvent('getphonenumber', evt, this);
    },
    onButtonError: function onButtonError(evt) {
      callSingleEvent('error', evt, this);
    },
    onButtonOpenSetting: function onButtonOpenSetting(evt) {
      callSingleEvent('opensetting', evt, this);
    },
    onButtonLaunchApp: function onButtonLaunchApp(evt) {
      callSingleEvent('launchapp', evt, this);
    }
  }
};

var editor = {
  name: 'editor',
  props: [{
    name: 'readOnly',
    get: function get(domNode) {
      return !!domNode.getAttribute('read-only');
    }
  }, {
    name: 'placeholder',
    get: function get(domNode) {
      return domNode.getAttribute('placeholder') || '';
    }
  }, {
    name: 'showImgSize',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-img-size');
    }
  }, {
    name: 'showImgToolbar',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-img-toolbar');
    }
  }, {
    name: 'showImgResize',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-img-resize');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onEditorReady: function onEditorReady(evt) {
      callSingleEvent('ready', evt, this);
    },
    onEditorFocus: function onEditorFocus(evt) {
      callSingleEvent('focus', evt, this);
    },
    onEditorBlur: function onEditorBlur(evt) {
      callSingleEvent('blur', evt, this);
    },
    onEditorInput: function onEditorInput(evt) {
      callSingleEvent('input', evt, this);
    },
    onEditorStatusChange: function onEditorStatusChange(evt) {
      callSingleEvent('statuschange', evt, this);
    }
  }
};

var form = {
  name: 'form',
  props: [{
    name: 'reportSubmit',
    get: function get(domNode) {
      return !!domNode.getAttribute('report-submit');
    }
  }, {
    name: 'reportSubmitTimeout',
    get: function get(domNode) {
      return +domNode.getAttribute('report-submit-timeout') || 0;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onFormSubmit: function onFormSubmit(evt) {
      callSingleEvent('submit', evt, this);
    },
    onFormReset: function onFormReset(evt) {
      callSingleEvent('reset', evt, this);
    }
  }
};

var label = {
  name: 'label',
  props: [{
    name: 'for',
    get: function get(domNode) {
      return domNode.getAttribute('for') || '';
    }
  }],
  handles: {}
};

// Check if the event passes through a node
function checkEventAccessDomNode(evt, domNode, dest) {
  var targetNode = dest || domNode.ownerDocument.body;
  var target = evt.target;
  if (domNode === targetNode) return true;

  while (target && target !== dest) {
    if (target === domNode) return true;
    target = target.parentNode;
  }

  return false;
}

// Find the closest parent node
function findParentNode(domNode, tagName) {
  var checkParentNode = function checkParentNode(parentNode, tagName) {
    if (!parentNode) return false;
    if (parentNode.tagName === tagName) return true;
    if (parentNode.tagName === 'BUILTIN-COMPONENT' && parentNode.behavior === tagName.toLowerCase()) return true;
    return false;
  };

  var parentNode = domNode.parentNode;
  if (checkParentNode(parentNode, tagName)) return parentNode;

  while (parentNode && parentNode.tagName !== tagName) {
    parentNode = parentNode.parentNode;
    if (checkParentNode(parentNode, tagName)) return parentNode;
  }

  return null;
}

var Event$1 = render.$$adapter.Event,
    EventTarget = render.$$adapter.EventTarget; // Call simple node event, no bubbling

function callSimpleEvent (eventName, evt, domNode) {
  if (!domNode) return;
  EventTarget.$$process(domNode, new Event$1({
    name: eventName,
    target: domNode,
    eventPhase: Event$1.AT_TARGET,
    detail: evt && evt.detail,
    $$extra: evt && evt.extra,
    bubbles: false
  }));
}

// eslint-disable-next-line import/no-extraneous-dependencies
var _render$$$adapter = render__default.$$adapter,
    cache = _render$$$adapter.cache,
    EventTarget$1 = _render$$$adapter.EventTarget;
function callEvent (eventName, evt, extra, pageId) {
  var originNodeId = evt.currentTarget.dataset.privateNodeId;
  var originNode = cache.getNode(pageId, originNodeId);
  if (!originNode) return;
  EventTarget$1.$$process(originNode, eventName, evt, extra, function (domNode, evt, isCapture) {
    {
      setTimeout(function () {
        if (evt.cancelable) return;
        var window = cache.getWindow();

        if (domNode.tagName === 'LABEL' && evt.type === 'click' && !isCapture) {
          var forValue = domNode.getAttribute('for');
          var targetDomNode;

          if (forValue) {
            targetDomNode = window.document.getElementById(forValue);
          } else {
            targetDomNode = domNode.querySelector('input');
            if (!targetDomNode) targetDomNode = domNode.querySelector('builtIn-component[behavior=switch]');
          }

          if (!targetDomNode || !!targetDomNode.getAttribute('disabled')) return;

          if (targetDomNode.tagName === 'INPUT') {
            if (checkEventAccessDomNode(evt, targetDomNode, domNode)) return;
            targetDomNode.focus();
          } else if (targetDomNode.tagName === 'BUILTIN-COMPONENT') {
            if (checkEventAccessDomNode(evt, targetDomNode, domNode)) return;
            var behavior = targetDomNode.behavior;

            if (behavior === 'switch') {
              var checked = !targetDomNode.getAttribute('checked');
              targetDomNode.setAttribute('checked', checked);
              callSimpleEvent('change', {
                detail: {
                  value: checked
                }
              }, targetDomNode);
            }
          }
        } else if ((domNode.tagName === 'BUTTON' || domNode.tagName === 'BUILTIN-COMPONENT' && domNode.behavior === 'button') && evt.type === 'click' && !isCapture) {
          var type = domNode.tagName === 'BUTTON' ? domNode.getAttribute('type') : domNode.getAttribute('form-type');
          var formAttr = domNode.getAttribute('form');
          var form = formAttr ? window.document.getElementById(formAttr) : findParentNode(domNode, 'FORM');
          if (!form) return;
          if (type !== 'submit' && type !== 'reset') return;
          var inputList = form.querySelectorAll('input[name]');
          var textareaList = form.querySelectorAll('textarea[name]');
          var switchList = form.querySelectorAll('builtin-component[behavior=switch]').filter(function (item) {
            return !!item.getAttribute('name');
          });
          var sliderList = form.querySelectorAll('builtin-component[behavior=slider]').filter(function (item) {
            return !!item.getAttribute('name');
          });
          var pickerList = form.querySelectorAll('builtin-component[behavior=picker]').filter(function (item) {
            return !!item.getAttribute('name');
          });

          if (type === 'submit') {
            var formData = {};

            if (inputList.length) {
              inputList.forEach(function (item) {
                formData[item.name] = item.value;
              });
            }

            if (textareaList.length) textareaList.forEach(function (item) {
              return formData[item.name] = item.value;
            });
            if (switchList.length) switchList.forEach(function (item) {
              return formData[item.getAttribute('name')] = !!item.getAttribute('checked');
            });
            if (sliderList.length) sliderList.forEach(function (item) {
              return formData[item.getAttribute('name')] = +item.getAttribute('value') || 0;
            });
            if (pickerList.length) pickerList.forEach(function (item) {
              return formData[item.getAttribute('name')] = item.getAttribute('value');
            });
            var detail = {
              value: formData
            };

            if (form._formId) {
              detail.formId = form._formId;
              form._formId = null;
            }

            callSimpleEvent('submit', {
              detail: detail,
              extra: {
                $$from: 'button'
              }
            }, form);
          } else if (type === 'reset') {
            if (inputList.length) {
              inputList.forEach(function (item) {
                item.setAttribute('value', '');
              });
            }

            if (textareaList.length) textareaList.forEach(function (item) {
              return item.setAttribute('value', '');
            });
            if (switchList.length) switchList.forEach(function (item) {
              return item.setAttribute('checked', undefined);
            });
            if (sliderList.length) sliderList.forEach(function (item) {
              return item.setAttribute('value', undefined);
            });
            if (pickerList.length) pickerList.forEach(function (item) {
              return item.setAttribute('value', undefined);
            });
            callSimpleEvent('reset', {
              extra: {
                $$from: 'button'
              }
            }, form);
          }
        }
      }, 0);
    }
  });
}

var cache$1 = render__default.$$adapter.cache;
var input = {
  name: 'input',
  props: [{
    name: 'value',
    canBeUserChanged: true,
    get: function get(domNode) {
      return domNode.value || '';
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'type',
    get: function get(domNode) {
      var value = domNode.type || 'text';
      return value !== 'password' ? value : 'text';
    }
  }, {
    name: 'password',
    get: function get(domNode) {
      return domNode.type !== 'password' ? !!domNode.getAttribute('password') : true;
    }
  }, {
    name: 'placeholder',
    get: function get(domNode) {
      return domNode.placeholder;
    }
  }, {
    name: 'placeholderColor',
    get: function get(domNode) {
      return domNode.getAttribute('placeholderColor') || '#999999';
    }
  }, {
    name: 'placeholderStyle',
    get: function get(domNode) {
      return domNode.getAttribute('placeholder-style') || '';
    }
  }, {
    name: 'placeholderClass',
    get: function get(domNode) {
      return domNode.getAttribute('placeholder-class') || 'input-placeholder';
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return domNode.disabled || domNode.readOnly;
    }
  }, {
    name: 'maxlength',
    get: function get(domNode) {
      var value = parseInt(domNode.maxlength, 10);
      return !isNaN(value) ? value : 140;
    }
  }, {
    name: 'cursorSpacing',
    get: function get(domNode) {
      return +domNode.getAttribute('cursor-spacing') || 0;
    }
  }, {
    name: 'autoFocus',
    get: function get(domNode) {
      return !!domNode.getAttribute('autofocus');
    }
  }, {
    name: 'focus',
    canBeUserChanged: true,
    get: function get(domNode) {
      return !!domNode.getAttribute('focus');
    }
  }, {
    name: 'confirmType',
    get: function get(domNode) {
      return domNode.getAttribute('confirm-type') || 'done';
    }
  }, {
    name: 'confirmHold',
    get: function get(domNode) {
      return !!domNode.getAttribute('confirm-hold');
    }
  }, {
    name: 'cursor',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('cursor'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'selectionStart',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('selection-start'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'selectionEnd',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('selection-end'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'adjustPosition',
    get: function get(domNode) {
      var value = domNode.getAttribute('adjust-position');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'checked',
    canBeUserChanged: true,
    get: function get(domNode) {
      return !!domNode.getAttribute('checked');
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '#09BB07';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }, {
    name: 'controlled',
    get: function get(domNode) {
      return !!domNode.getAttribute('controlled');
    }
  }],
  handles: {
    onInputInput: function onInputInput(evt) {
      var domNode = this.getDomNodeFromEvt('input', evt);
      if (!domNode) return;
      var value = '' + evt.detail.value;
      domNode.$$setAttributeWithoutUpdate('value', value);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.value = value;
      callEvent('input', evt, null, this.pageId);
    },
    onInputFocus: function onInputFocus(evt) {
      var domNode = this.getDomNodeFromEvt('focus', evt);
      if (!domNode) return;
      domNode.__inputOldValue = domNode.value;
      domNode.$$setAttributeWithoutUpdate('focus', true);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.focus = true;
      callSimpleEvent('focus', evt, domNode);
    },
    onInputBlur: function onInputBlur(evt) {
      var domNode = this.getDomNodeFromEvt('blur', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('focus', false);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.focus = false;

      if (this.__inputOldValue !== undefined && domNode.value !== this.__inputOldValue) {
        this.__inputOldValue = undefined;
        callEvent('change', evt, null, this.pageId);
      }

      callSimpleEvent('blur', evt, domNode);
    },
    onInputConfirm: function onInputConfirm(evt) {
      var domNode = this.getDomNodeFromEvt('confirm', evt);
      callSimpleEvent('confirm', evt, domNode);
    },
    onInputKeyBoardHeightChange: function onInputKeyBoardHeightChange(evt) {
      callSingleEvent('keyboardheightchange', evt, this);
    },
    onRadioChange: function onRadioChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      var window = cache$1.getWindow();
      var value = evt.detail.value;
      var name = domNode.name;

      if (value === domNode.value) {
        domNode.$$setAttributeWithoutUpdate('checked', true);
        domNode.__oldValues = domNode.__oldValues || {};
        domNode.__oldValues.checked = true;
        var otherDomNodes = window.document.querySelectorAll("input[name=" + name + "]") || [];

        for (var _iterator = _createForOfIteratorHelperLoose(otherDomNodes), _step; !(_step = _iterator()).done;) {
          var otherDomNode = _step.value;

          if (otherDomNode.type === 'radio' && otherDomNode !== domNode) {
            otherDomNode.$$setAttributeWithoutUpdate('checked', false);
            otherDomNode.__oldValues = otherDomNode.__oldValues || {};
            otherDomNode.__oldValues.checked = false;
          }
        }
      }

      callEvent('input', evt, null, this.pageId);
      callEvent('change', evt, null, this.pageId);
    },
    onCheckboxChange: function onCheckboxChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      var value = evt.detail.value || [];

      if (value.indexOf(domNode.value) >= 0) {
        domNode.$$setAttributeWithoutUpdate('checked', true);
        domNode.__oldValues = domNode.__oldValues || {};
        domNode.__oldValues.checked = true;
      } else {
        domNode.$$setAttributeWithoutUpdate('checked', false);
        domNode.__oldValues = domNode.__oldValues || {};
        domNode.__oldValues.checked = false;
      }

      callEvent('change', evt, null, this.pageId);
    },
    onCheckboxItemChange: function onCheckboxItemChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      var value = evt.detail.value || false;
      domNode.setAttribute('checked', value);
      callEvent('change', evt, null, this.pageId);
    }
  }
};

var radioGroup = {
  name: 'radio-group',
  props: [{
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }],
  handles: {
    onRadioChange: function onRadioChange(evt) {
      callSingleEvent('change', evt, this);
    }
  }
};

var radio = {
  name: 'radio',
  props: [{
    name: 'value',
    get: function get(domNode) {
      return domNode.getAttribute('value') || '';
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'checked',
    get: function get(domNode) {
      return domNode.getAttribute('checked') || false;
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return domNode.getAttribute('disabled') || false;
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '';
    }
  }],
  handles: {
    onRadioChange: function onRadioChange(evt) {
      callSingleEvent('change', evt, this);
    }
  }
};

var checkboxGroup = {
  name: 'checkbox-group',
  props: [{
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }],
  handles: {
    onCheckboxChange: function onCheckboxChange(evt) {
      callSingleEvent('change', evt, this);
    }
  }
};

var checkbox = {
  name: 'checkbox',
  props: [{
    name: 'value',
    get: function get(domNode) {
      return domNode.getAttribute('value') || '';
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'checked',
    get: function get(domNode) {
      return domNode.getAttribute('checked') || false;
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return domNode.getAttribute('disabled') || false;
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '';
    }
  }],
  handles: {
    onCheckboxItemChange: function onCheckboxItemChange(evt) {
      callSingleEvent('change', evt, this);
    }
  }
};

// eslint-disable-next-line import/no-extraneous-dependencies
var picker = {
  name: 'picker',
  props: [{
    name: 'disabled',
    get: function get(domNode) {
      return !!domNode.getAttribute('disabled');
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'range',
    get: function get(domNode) {
      var value = domNode.getAttribute('range');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'rangeKey',
    get: function get(domNode) {
      return domNode.getAttribute('range-key') || '';
    }
  }, {
    name: 'value',
    canBeUserChanged: true,
    get: function get(domNode) {
      var mode = domNode.getAttribute('mode') || 'selector';
      var value = domNode.getAttribute('value');

      if (mode === 'selector' || mode === 'multiSelector') {
        return +value || 0;
      } else if (mode === 'time') {
        return value || '';
      } else if (mode === 'date') {
        return value || '0';
      } else if (mode === 'region') {
        return value || [];
      }

      return value;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onPickerChange: function onPickerChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('value', evt.detail.value);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.value = evt.detail.value;
      callSingleEvent('change', evt, this);
    },
    onPickerCancel: function onPickerCancel(evt) {
      callSingleEvent('cancel', evt, this);
    }
  }
};

{
  picker.props.concat([{
    name: 'mode',
    get: function get(domNode) {
      return domNode.getAttribute('mode') || 'selector';
    }
  }, {
    name: 'start',
    get: function get(domNode) {
      return domNode.getAttribute('start') || '';
    }
  }, {
    name: 'end',
    get: function get(domNode) {
      return domNode.getAttribute('end') || '';
    }
  }, {
    name: 'fields',
    get: function get(domNode) {
      return domNode.getAttribute('fields') || 'day';
    }
  }, {
    name: 'customItem',
    get: function get(domNode) {
      return domNode.getAttribute('custom-item') || '';
    }
  }]);

  picker.handles.onPickerColumnChange = function (evt) {
    callSingleEvent('columnchange', evt, this);
  };
}

var pickerView = {
  name: 'picker-view',
  props: [{
    name: 'value',
    canBeUserChanged: true,
    get: function get(domNode) {
      var value = domNode.getAttribute('value');
      if (typeof value === 'string') value = value.split(',').map(function (item) {
        return parseInt(item, 10);
      });
      return value !== undefined ? value : [];
    }
  }, {
    name: 'indicatorStyle',
    get: function get(domNode) {
      return domNode.getAttribute('indicator-style') || '';
    }
  }, {
    name: 'indicatorClass',
    get: function get(domNode) {
      return domNode.getAttribute('indicator-class') || '';
    }
  }, {
    name: 'maskStyle',
    get: function get(domNode) {
      return domNode.getAttribute('mask-style') || '';
    }
  }, {
    name: 'maskClass',
    get: function get(domNode) {
      return domNode.getAttribute('mask-class') || '';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onPickerViewChange: function onPickerViewChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('value', evt.detail.value);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.value = evt.detail.value;
      callSingleEvent('change', evt, this);
    },
    onPickerViewPickstart: function onPickerViewPickstart(evt) {
      callSingleEvent('pickstart', evt, this);
    },
    onPickerViewPickend: function onPickerViewPickend(evt) {
      callSingleEvent('pickend', evt, this);
    }
  }
};

var slider = {
  name: 'slider',
  props: [{
    name: 'min',
    get: function get(domNode) {
      return +domNode.getAttribute('min') || 0;
    }
  }, {
    name: 'max',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('max'), 10);
      return !isNaN(value) ? value : 100;
    }
  }, {
    name: 'step',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('step'), 10);
      return !isNaN(value) ? value : 1;
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return !!domNode.getAttribute('disabled');
    }
  }, {
    name: 'value',
    canBeUserChanged: true,
    get: function get(domNode) {
      return +domNode.getAttribute('value') || 0;
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '#e9e9e9';
    }
  }, {
    name: 'selectedColor',
    get: function get(domNode) {
      return domNode.getAttribute('selected-color') || '#1aad19';
    }
  }, {
    name: 'activeColor',
    get: function get(domNode) {
      return domNode.getAttribute('active-color') || '#1aad19';
    }
  }, {
    name: 'backgroundColor',
    get: function get(domNode) {
      return domNode.getAttribute('background-color') || '#e9e9e9';
    }
  }, {
    name: 'blockSize',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('block-size'), 10);
      return !isNaN(value) ? value : 28;
    }
  }, {
    name: 'blockColor',
    get: function get(domNode) {
      return domNode.getAttribute('block-color') || '#ffffff';
    }
  }, {
    name: 'showValue',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-value');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onSliderChange: function onSliderChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('value', evt.detail.value);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.value = evt.detail.value;
      callSingleEvent('change', evt, this);
    },
    onSliderChanging: function onSliderChanging(evt) {
      callSingleEvent('changing', evt, this);
    }
  }
};

var switchCom = {
  name: 'switch',
  props: [{
    name: 'checked',
    canBeUserChanged: true,
    get: function get(domNode) {
      return !!domNode.getAttribute('checked');
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return !!domNode.getAttribute('disabled');
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'type',
    get: function get(domNode) {
      return domNode.getAttribute('type') || 'switch';
    }
  }, {
    name: 'color',
    get: function get(domNode) {
      return domNode.getAttribute('color') || '#04BE02';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onSwitchChange: function onSwitchChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('checked', evt.detail.value);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.checked = evt.detail.value;
      callSingleEvent('change', evt, this);
    }
  }
};

var textarea = {
  name: 'textarea',
  props: [{
    name: 'value',
    canBeUserChanged: true,
    get: function get(domNode) {
      return domNode.value || '';
    }
  }, {
    name: 'name',
    get: function get(domNode) {
      return domNode.getAttribute('name') || '';
    }
  }, {
    name: 'placeholder',
    get: function get(domNode) {
      return domNode.placeholder;
    }
  }, {
    name: 'placeholderColor',
    get: function get(domNode) {
      return domNode.getAttribute('placeholderColor') || '#999999';
    }
  }, {
    name: 'placeholderStyle',
    get: function get(domNode) {
      return domNode.getAttribute('placeholder-style') || '';
    }
  }, {
    name: 'placeholderClass',
    get: function get(domNode) {
      return domNode.getAttribute('placeholder-class') || 'input-placeholder';
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return domNode.disabled || domNode.readOnly;
    }
  }, {
    name: 'maxlength',
    get: function get(domNode) {
      var value = parseInt(domNode.maxlength, 10);
      return !isNaN(value) ? value : 140;
    }
  }, {
    name: 'autoFocus',
    get: function get(domNode) {
      return !!domNode.getAttribute('autofocus');
    }
  }, {
    name: 'focus',
    get: function get(domNode) {
      return !!domNode.getAttribute('focus');
    }
  }, {
    name: 'autoHeight',
    get: function get(domNode) {
      return !!domNode.getAttribute('auto-height');
    }
  }, {
    name: 'fixed',
    get: function get(domNode) {
      return !!domNode.getAttribute('fixed');
    }
  }, {
    name: 'cursorSpacing',
    get: function get(domNode) {
      return +domNode.getAttribute('cursor-spacing') || 0;
    }
  }, {
    name: 'cursor',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('cursor'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'showConfirmBar',
    get: function get(domNode) {
      var value = domNode.getAttribute('show-confirm-bar');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'selectionStart',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('selection-start'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'selectionEnd',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('selection-end'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'adjustPosition',
    get: function get(domNode) {
      var value = domNode.getAttribute('adjust-position');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }, {
    name: 'controlled',
    get: function get(domNode) {
      return !!domNode.getAttribute('controlled');
    }
  }],
  handles: {
    onTextareaFocus: function onTextareaFocus(evt) {
      var domNode = this.getDomNodeFromEvt('focus', evt);
      if (!domNode) return;
      domNode.__textareaOldValue = domNode.value;
      domNode.$$setAttributeWithoutUpdate('focus', true);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.focus = true;
      callSimpleEvent('focus', evt, domNode);
    },
    onTextareaBlur: function onTextareaBlur(evt) {
      var domNode = this.getDomNodeFromEvt('blur', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('focus', false);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.focus = false;

      if (this.__textareaOldValue !== undefined && domNode.value !== this.__textareaOldValue) {
        this.__textareaOldValue = undefined;
        callEvent('change', evt);
      }

      callSimpleEvent('blur', evt, domNode);
    },
    onTextareaLineChange: function onTextareaLineChange(evt) {
      var domNode = this.getDomNodeFromEvt('linechange', evt);
      callSimpleEvent('linechange', evt, domNode);
    },
    onTextareaInput: function onTextareaInput(evt) {
      var domNode = this.getDomNodeFromEvt('blur', evt);
      if (!domNode) return;
      var value = '' + evt.detail.value;
      domNode.$$setAttributeWithoutUpdate('value', value);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.value = value;
      callEvent('input', evt, null, this.pageId);
    },
    onTextareaConfirm: function onTextareaConfirm(evt) {
      var domNode = this.getDomNodeFromEvt('confirm', evt);
      callSimpleEvent('confirm', evt, domNode);
    },
    onTextareaKeyBoardHeightChange: function onTextareaKeyBoardHeightChange(evt) {
      callSingleEvent('keyboardheightchange', evt, this);
    }
  }
};

/* global isWeChatMiniProgram */
var props$1 = [{
  name: 'openType',
  get: function get(domNode) {
    return domNode.getAttribute('open-type') || 'navigate';
  }
}, {
  name: 'hoverClass',
  get: function get(domNode) {
    return domNode.getAttribute('hover-class') || 'none';
  }
}, {
  name: 'hoverStartTime',
  get: function get(domNode) {
    var value = parseInt(domNode.getAttribute('hover-start-time'), 10);
    return !isNaN(value) ? value : 50;
  }
}, {
  name: 'hoverStayTime',
  get: function get(domNode) {
    var value = parseInt(domNode.getAttribute('hover-stay-time'), 10);
    return !isNaN(value) ? value : 600;
  }
}, {
  name: 'url',
  get: function get(domNode) {
    return domNode.getAttribute('url') || '';
  }
}, {
  name: 'animation',
  get: function get(domNode) {
    return domNode.getAttribute('animation');
  }
}];

{
  props$1 = props$1.concat([{
    name: 'target',
    get: function get(domNode) {
      return domNode.getAttribute('target') || 'self';
    }
  }, {
    name: 'delta',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('delta'), 10);
      return !isNaN(value) ? value : 1;
    }
  }, {
    name: 'appId',
    get: function get(domNode) {
      return domNode.getAttribute('app-id') || '';
    }
  }, {
    name: 'path',
    get: function get(domNode) {
      return domNode.getAttribute('path') || '';
    }
  }, {
    name: 'extraData',
    get: function get(domNode) {
      return domNode.getAttribute('extra-data') || {};
    }
  }, {
    name: 'version',
    get: function get(domNode) {
      return domNode.getAttribute('version') || 'release';
    }
  }, {
    name: 'hoverStopPropagation',
    get: function get(domNode) {
      return !!domNode.getAttribute('hover-stop-propagation');
    }
  }]);
}

var navigator = {
  name: 'navigator',
  props: props$1,
  handles: {
    onNavigatorSuccess: function onNavigatorSuccess(evt) {
      callSingleEvent('success', evt, this);
    },
    onNavigatorFail: function onNavigatorFail(evt) {
      callSingleEvent('fail', evt, this);
    },
    onNavigatorComplete: function onNavigatorComplete(evt) {
      callSingleEvent('complete', evt, this);
    }
  }
};

var camera = {
  name: 'camera',
  props: [{
    name: 'mode',
    get: function get(domNode) {
      return domNode.getAttribute('mode') || 'normal';
    }
  }, {
    name: 'devicePosition',
    get: function get(domNode) {
      return domNode.getAttribute('device-position') || 'back';
    }
  }, {
    name: 'flash',
    get: function get(domNode) {
      return domNode.getAttribute('flash') || 'auto';
    }
  }, {
    name: 'frameSize',
    get: function get(domNode) {
      return domNode.getAttribute('frame-size') || 'medium';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onCameraStop: function onCameraStop(evt) {
      callSingleEvent('stop', evt, this);
    },
    onCameraError: function onCameraError(evt) {
      callSingleEvent('error', evt, this);
    },
    onCameraInitDone: function onCameraInitDone(evt) {
      callSingleEvent('initdone', evt, this);
    },
    onCameraScanCode: function onCameraScanCode(evt) {
      callSingleEvent('scancode', evt, this);
    }
  }
};

var image = {
  name: 'image',
  props: [{
    name: 'renderingMode',
    get: function get(domNode) {
      return domNode.getAttribute('rendering-mode') || '';
    }
  }, {
    name: 'src',
    get: function get(domNode) {
      return domNode.src;
    }
  }, {
    name: 'mode',
    get: function get(domNode) {
      return domNode.getAttribute('mode') || 'scaleToFill';
    }
  }, {
    name: 'lazyLoad',
    get: function get(domNode) {
      return !!domNode.getAttribute('lazy-load');
    }
  }, {
    name: 'showMenuByLongpress',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-menu-by-longpress');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onImageLoad: function onImageLoad(evt) {
      callSingleEvent('load', evt, this);
    },
    onImageError: function onImageError(evt) {
      callSingleEvent('error', evt, this);
    }
  }
};

var livePlayer = {
  name: 'live-player',
  props: [{
    name: 'src',
    get: function get(domNode) {
      return domNode.src;
    }
  }, {
    name: 'mode',
    get: function get(domNode) {
      return domNode.getAttribute('mode') || 'live';
    }
  }, {
    name: 'autoplay',
    get: function get(domNode) {
      return !!domNode.getAttribute('autoplay');
    }
  }, {
    name: 'muted',
    get: function get(domNode) {
      return !!domNode.getAttribute('muted');
    }
  }, {
    name: 'orientation',
    get: function get(domNode) {
      return domNode.getAttribute('orientation') || 'vertical';
    }
  }, {
    name: 'objectFit',
    get: function get(domNode) {
      return domNode.getAttribute('object-fit') || 'contain';
    }
  }, {
    name: 'backgroundMute',
    get: function get(domNode) {
      return !!domNode.getAttribute('background-mute');
    }
  }, {
    name: 'minCache',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('min-cache'), 10);
      return !isNaN(value) ? value : 1;
    }
  }, {
    name: 'maxCache',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('max-cache'), 10);
      return !isNaN(value) ? value : 3;
    }
  }, {
    name: 'soundMode',
    get: function get(domNode) {
      return domNode.getAttribute('sound-mode') || 'speaker';
    }
  }, {
    name: 'autoPauseIfNavigate',
    get: function get(domNode) {
      var value = domNode.getAttribute('auto-pause-if-navigate');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'autoPauseIfOpenNative',
    get: function get(domNode) {
      var value = domNode.getAttribute('auto-pause-if-open-native');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onLivePlayerStateChange: function onLivePlayerStateChange(evt) {
      callSingleEvent('statechange', evt, this);
    },
    onLivePlayerFullScreenChange: function onLivePlayerFullScreenChange(evt) {
      callSingleEvent('fullscreenchange', evt, this);
    },
    onLivePlayerNetStatus: function onLivePlayerNetStatus(evt) {
      callSingleEvent('netstatus', evt, this);
    }
  }
};

var livePusher = {
  name: 'live-pusher',
  props: [{
    name: 'url',
    get: function get(domNode) {
      return domNode.getAttribute('url');
    }
  }, {
    name: 'mode',
    get: function get(domNode) {
      return domNode.getAttribute('mode') || 'RTC';
    }
  }, {
    name: 'autopush',
    get: function get(domNode) {
      return !!domNode.getAttribute('autopush');
    }
  }, {
    name: 'muted',
    get: function get(domNode) {
      return !!domNode.getAttribute('muted');
    }
  }, {
    name: 'enableCamera',
    get: function get(domNode) {
      var value = domNode.getAttribute('enable-camera');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'autoFocus',
    get: function get(domNode) {
      var value = domNode.getAttribute('auto-focus');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'orientation',
    get: function get(domNode) {
      return domNode.getAttribute('orientation') || 'vertical';
    }
  }, {
    name: 'beauty',
    get: function get(domNode) {
      return +domNode.getAttribute('beauty') || 0;
    }
  }, {
    name: 'whiteness',
    get: function get(domNode) {
      return +domNode.getAttribute('whiteness') || 0;
    }
  }, {
    name: 'aspect',
    get: function get(domNode) {
      return domNode.getAttribute('aspect') || '9:16';
    }
  }, {
    name: 'minBitrate',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('min-bitrate'), 10);
      return !isNaN(value) ? value : 200;
    }
  }, {
    name: 'maxBitrate',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('max-bitrate'), 10);
      return !isNaN(value) ? value : 1000;
    }
  }, {
    name: 'waitingImage',
    get: function get(domNode) {
      return domNode.getAttribute('waiting-image') || '';
    }
  }, {
    name: 'waitingImageHash',
    get: function get(domNode) {
      return domNode.getAttribute('waiting-image-hash') || '';
    }
  }, {
    name: 'zoom',
    get: function get(domNode) {
      return !!domNode.getAttribute('zoom');
    }
  }, {
    name: 'devicePosition',
    get: function get(domNode) {
      return domNode.getAttribute('device-position') || 'front';
    }
  }, {
    name: 'backgroundMute',
    get: function get(domNode) {
      return !!domNode.getAttribute('background-mute');
    }
  }, {
    name: 'mirror',
    get: function get(domNode) {
      return !!domNode.getAttribute('mirror');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onLivePusherStateChange: function onLivePusherStateChange(evt) {
      callSingleEvent('statechange', evt, this);
    },
    onLivePusherNetStatus: function onLivePusherNetStatus(evt) {
      callSingleEvent('netstatus', evt, this);
    },
    onLivePusherError: function onLivePusherError(evt) {
      callSingleEvent('error', evt, this);
    },
    onLivePusherBgmStart: function onLivePusherBgmStart(evt) {
      callSingleEvent('bgmstart', evt, this);
    },
    onLivePusherBgmProgress: function onLivePusherBgmProgress(evt) {
      callSingleEvent('bgmprogress', evt, this);
    },
    onLivePusherBgmComplete: function onLivePusherBgmComplete(evt) {
      callSingleEvent('bgmcomplete', evt, this);
    }
  }
};

var video = {
  name: 'video',
  props: [{
    name: 'src',
    get: function get(domNode) {
      return domNode.src;
    }
  }, {
    name: 'duration',
    get: function get(domNode) {
      return +domNode.getAttribute('duration') || 0;
    }
  }, {
    name: 'controls',
    get: function get(domNode) {
      return domNode.controls;
    }
  }, {
    name: 'danmuList',
    get: function get(domNode) {
      var value = domNode.getAttribute('danmu-list');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'danmuBtn',
    get: function get(domNode) {
      return !!domNode.getAttribute('danmu-btn');
    }
  }, {
    name: 'enableDanmu',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-danmu');
    }
  }, {
    name: 'autoplay',
    get: function get(domNode) {
      return domNode.autoplay;
    }
  }, {
    name: 'loop',
    get: function get(domNode) {
      return domNode.loop;
    }
  }, {
    name: 'muted',
    get: function get(domNode) {
      return domNode.muted;
    }
  }, {
    name: 'initialTime',
    get: function get(domNode) {
      return +domNode.getAttribute('initial-time') || 0;
    }
  }, {
    name: 'direction',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('direction'), 10);
      return !isNaN(value) ? value : -1;
    }
  }, {
    name: 'showProgress',
    get: function get(domNode) {
      var value = domNode.getAttribute('show-progress');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'showFullscreenBtn',
    get: function get(domNode) {
      var value = domNode.getAttribute('show-fullscreen-btn');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'showPlayBtn',
    get: function get(domNode) {
      var value = domNode.getAttribute('show-play-btn');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'showCenterPlayBtn',
    get: function get(domNode) {
      var value = domNode.getAttribute('show-center-play-btn');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'enableProgressGesture',
    get: function get(domNode) {
      var value = domNode.getAttribute('enable-progress-gesture');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'objectFit',
    get: function get(domNode) {
      return domNode.getAttribute('object-fit') || 'contain';
    }
  }, {
    name: 'poster',
    get: function get(domNode) {
      return domNode.poster;
    }
  }, {
    name: 'showMuteBtn',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-mute-btn');
    }
  }, {
    name: 'title',
    get: function get(domNode) {
      return domNode.getAttribute('title') || '';
    }
  }, {
    name: 'playBtnPosition',
    get: function get(domNode) {
      return domNode.getAttribute('play-btn-position') || 'bottom';
    }
  }, {
    name: 'enablePlayGesture',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-play-gesture');
    }
  }, {
    name: 'autoPauseIfNavigate',
    get: function get(domNode) {
      var value = domNode.getAttribute('auto-pause-if-navigate');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'autoPauseIfOpenNative',
    get: function get(domNode) {
      var value = domNode.getAttribute('auto-pause-if-open-native');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'vslideGesture',
    get: function get(domNode) {
      return !!domNode.getAttribute('vslide-gesture');
    }
  }, {
    name: 'vslideGestureInFullscreen',
    get: function get(domNode) {
      var value = domNode.getAttribute('vslide-gesture-in-fullscreen');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onVideoPlay: function onVideoPlay(evt) {
      callSingleEvent('play', evt, this);
    },
    onVideoPause: function onVideoPause(evt) {
      callSingleEvent('pause', evt, this);
    },
    onVideoEnded: function onVideoEnded(evt) {
      callSingleEvent('ended', evt, this);
    },
    onVideoTimeUpdate: function onVideoTimeUpdate(evt) {
      var domNode = this.getDomNodeFromEvt('timeupdate', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('currentTime', evt.detail.currentTime);
      callSingleEvent('timeupdate', evt, this);
    },
    onVideoFullScreenChange: function onVideoFullScreenChange(evt) {
      callSingleEvent('fullscreenchange', evt, this);
    },
    onVideoWaiting: function onVideoWaiting(evt) {
      callSingleEvent('waiting', evt, this);
    },
    onVideoError: function onVideoError(evt) {
      callSingleEvent('error', evt, this);
    },
    onVideoProgress: function onVideoProgress(evt) {
      var domNode = this.getDomNodeFromEvt('progress', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('buffered', evt.detail.buffered);
      callSingleEvent('progress', evt, this);
    }
  }
};

var map = {
  name: 'map',
  props: [{
    name: 'longitude',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('longitude'), 10);
      return !isNaN(value) ? value : 39.92;
    }
  }, {
    name: 'latitude',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('latitude'), 10);
      return !isNaN(value) ? value : 116.46;
    }
  }, {
    name: 'scale',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('scale'), 10);
      return !isNaN(value) ? value : 16;
    }
  }, {
    name: 'markers',
    get: function get(domNode) {
      var value = domNode.getAttribute('markers');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'polyline',
    get: function get(domNode) {
      var value = domNode.getAttribute('polyline');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'circles',
    get: function get(domNode) {
      var value = domNode.getAttribute('circles');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'controls',
    get: function get(domNode) {
      var value = domNode.getAttribute('controls');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'includePoints',
    get: function get(domNode) {
      var value = domNode.getAttribute('include-points');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'showLocation',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-location');
    }
  }, {
    name: 'polygons',
    get: function get(domNode) {
      var value = domNode.getAttribute('polygons');
      return value !== undefined ? value : [];
    }
  }, {
    name: 'subkey',
    get: function get(domNode) {
      return domNode.getAttribute('subkey') || '';
    }
  }, {
    name: 'layerStyle',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('layer-style'), 10);
      return !isNaN(value) ? value : 1;
    }
  }, {
    name: 'rotate',
    canBeUserChanged: true,
    get: function get(domNode) {
      return +domNode.getAttribute('rotate') || 0;
    }
  }, {
    name: 'skew',
    canBeUserChanged: true,
    get: function get(domNode) {
      return +domNode.getAttribute('skew') || 0;
    }
  }, {
    name: 'enable3D',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-3D');
    }
  }, {
    name: 'showCompass',
    get: function get(domNode) {
      return !!domNode.getAttribute('show-compass');
    }
  }, {
    name: 'enableOverlooking',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-overlooking');
    }
  }, {
    name: 'enableZoom',
    get: function get(domNode) {
      var value = domNode.getAttribute('enable-zoom');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'enableScroll',
    get: function get(domNode) {
      var value = domNode.getAttribute('enable-scroll');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'enableRotate',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-rotate');
    }
  }, {
    name: 'enableSatellite',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-satellite');
    }
  }, {
    name: 'enableTraffic',
    get: function get(domNode) {
      return !!domNode.getAttribute('enable-traffic');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onMapTap: function onMapTap(evt) {
      callSingleEvent('tap', evt, this);
    },
    onMapMarkerTap: function onMapMarkerTap(evt) {
      callSingleEvent('markertap', evt, this);
    },
    onMapControlTap: function onMapControlTap(evt) {
      callSingleEvent('controltap', evt, this);
    },
    onMapCalloutTap: function onMapCalloutTap(evt) {
      callSingleEvent('callouttap', evt, this);
    },
    onMapUpdated: function onMapUpdated(evt) {
      callSingleEvent('updated', evt, this);
    },
    onMapRegionChange: function onMapRegionChange(evt) {
      var domNode = this.getDomNodeFromEvt('regionchange', evt);
      if (!domNode) return;
      if (!evt.detail.causedBy) evt.detail.causedBy = evt.causedBy;

      if (evt.type === 'end' || evt.detail.type === 'end') {
        domNode.__oldValues = domNode.__oldValues || {};
        domNode.__oldValues.rotate = evt.detail.rotate;
        domNode.__oldValues.skew = evt.detail.skew;
      }

      callSingleEvent('regionchange', evt, this);
    },
    onMapPoiTap: function onMapPoiTap(evt) {
      callSingleEvent('poitap', evt, this);
    }
  }
};

var canvas = {
  name: 'canvas',
  props: [{
    name: 'type',
    get: function get(domNode) {
      return domNode.getAttribute('type') || '';
    }
  }, {
    name: 'canvasId',
    get: function get(domNode) {
      return domNode.getAttribute('canvas-id') || '';
    }
  }, {
    name: 'disableScroll',
    get: function get(domNode) {
      return !!domNode.getAttribute('disable-scroll');
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onCanvasTouchStart: function onCanvasTouchStart(evt) {
      callSingleEvent('canvastouchstart', evt, this);
    },
    onCanvasTouchMove: function onCanvasTouchMove(evt) {
      callSingleEvent('canvastouchmove', evt, this);
    },
    onCanvasTouchEnd: function onCanvasTouchEnd(evt) {
      callSingleEvent('canvastouchend', evt, this);
    },
    onCanvasTouchCancel: function onCanvasTouchCancel(evt) {
      callSingleEvent('canvastouchcancel', evt, this);
    },
    onCanvasLongTap: function onCanvasLongTap(evt) {
      callSingleEvent('longtap', evt, this);
    },
    onCanvasError: function onCanvasError(evt) {
      callSingleEvent('error', evt, this);
    }
  }
};

var ad = {
  name: 'ad',
  props: [{
    name: 'unitId',
    get: function get(domNode) {
      return domNode.getAttribute('unit-id') || '';
    }
  }, {
    name: 'adIntervals',
    get: function get(domNode) {
      return +domNode.getAttribute('ad-intervals') || 0;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onAdLoad: function onAdLoad(evt) {
      callSingleEvent('load', evt, this);
    },
    onAdError: function onAdError(evt) {
      callSingleEvent('error', evt, this);
    },
    onAdClose: function onAdClose(evt) {
      callSingleEvent('close', evt, this);
    }
  }
};

var officialAccount = {
  name: 'official-account',
  props: [{
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onOfficialAccountLoad: function onOfficialAccountLoad(evt) {
      callSingleEvent('load', evt, this);
    },
    onOfficialAccountError: function onOfficialAccountError(evt) {
      callSingleEvent('error', evt, this);
    }
  }
};

var openData = {
  name: 'open-data',
  props: [{
    name: 'type',
    get: function get(domNode) {
      return domNode.getAttribute('type') || '';
    }
  }, {
    name: 'openGid',
    get: function get(domNode) {
      return domNode.getAttribute('open-gid') || '';
    }
  }, {
    name: 'lang',
    get: function get(domNode) {
      return domNode.getAttribute('lang') || 'en';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

var webView = {
  name: 'web-view',
  props: [{
    name: 'src',
    get: function get(domNode) {
      return domNode.src;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onWebviewMessage: function onWebviewMessage(evt) {
      callSingleEvent('message', evt, this);
    },
    onWebviewLoad: function onWebviewLoad(evt) {
      callSingleEvent('load', evt, this);
    },
    onWebviewError: function onWebviewError(evt) {
      callSingleEvent('error', evt, this);
    }
  }
};

var movableView = {
  name: 'movable-view',
  props: [{
    name: 'direction',
    get: function get(domNode) {
      return domNode.getAttribute('direction') || 'none';
    }
  }, {
    name: 'inertia',
    get: function get(domNode) {
      return !!domNode.getAttribute('inertia');
    }
  }, {
    name: 'outOfBounds',
    get: function get(domNode) {
      return !!domNode.getAttribute('out-of-bounds');
    }
  }, {
    name: 'x',
    canBeUserChanged: true,
    get: function get(domNode) {
      return +domNode.getAttribute('x') || 0;
    }
  }, {
    name: 'y',
    canBeUserChanged: true,
    get: function get(domNode) {
      return +domNode.getAttribute('y') || 0;
    }
  }, {
    name: 'damping',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('damping'), 10);
      return !isNaN(value) ? value : 20;
    }
  }, {
    name: 'friction',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('friction'), 10);
      return !isNaN(value) ? value : 2;
    }
  }, {
    name: 'disabled',
    get: function get(domNode) {
      return !!domNode.getAttribute('disabled');
    }
  }, {
    name: 'scale',
    canBeUserChanged: true,
    get: function get(domNode) {
      return !!domNode.getAttribute('scale');
    }
  }, {
    name: 'scaleMin',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('scale-min'), 10);
      return !isNaN(value) ? value : 0.5;
    }
  }, {
    name: 'scaleMax',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('scale-max'), 10);
      return !isNaN(value) ? value : 10;
    }
  }, {
    name: 'scaleValue',
    get: function get(domNode) {
      var value = parseInt(domNode.getAttribute('scale-value'), 10);
      return !isNaN(value) ? value : 1;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      var value = domNode.getAttribute('animation');
      return value !== undefined ? !!value : true;
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {
    onMovableViewChange: function onMovableViewChange(evt) {
      var domNode = this.getDomNodeFromEvt('change', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('x', evt.detail.x);
      domNode.$$setAttributeWithoutUpdate('y', evt.detail.y);
      domNode.__oldValues = domNode.__oldValues || {};
      domNode.__oldValues.x = evt.detail.x;
      domNode.__oldValues.y = evt.detail.y;
      callSingleEvent('change', evt, this);
    },
    onMovableViewScale: function onMovableViewScale(evt) {
      var domNode = this.getDomNodeFromEvt('scale', evt);
      if (!domNode) return;
      domNode.$$setAttributeWithoutUpdate('x', evt.detail.x);
      domNode.$$setAttributeWithoutUpdate('y', evt.detail.y);
      domNode.$$setAttributeWithoutUpdate('scale-value', evt.detail.scale);
      callSingleEvent('scale', evt, this);
    },
    onMovableViewHtouchmove: function onMovableViewHtouchmove(evt) {
      callSingleEvent('htouchmove', evt, this);
    },
    onMovableViewVtouchmove: function onMovableViewVtouchmove(evt) {
      callSingleEvent('vtouchmove', evt, this);
    }
  }
};

var swiperItem = {
  name: 'swiper-item',
  props: [{
    name: 'itemId',
    get: function get(domNode) {
      return domNode.getAttribute('item-id') || '';
    }
  }, {
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

var pickerViewColumn = {
  name: 'picker-view-column',
  props: [{
    name: 'animation',
    get: function get(domNode) {
      return domNode.getAttribute('animation');
    }
  }],
  handles: {}
};

// Components
var components = [coverImage, coverView, movableArea, ScrollView, swiper, view, icon, progress, text, richText, button, editor, form, label, input, radioGroup, radio, checkboxGroup, checkbox, picker, pickerView, slider, switchCom, textarea, navigator, camera, image, livePlayer, livePusher, video, map, canvas, ad, officialAccount, openData, webView];
var subComponents = [movableView, swiperItem, pickerViewColumn];
var componentNameMap = {};
var propsMap = {};
var handlesMap = {};
components.forEach(function (_ref) {
  var name = _ref.name,
      props = _ref.props,
      handles = _ref.handles;
  componentNameMap[name] = name;
  propsMap[name] = props;
  Object.assign(handlesMap, handles);
});
subComponents.forEach(function (_ref2) {
  var name = _ref2.name,
      props = _ref2.props,
      handles = _ref2.handles;
  propsMap[name] = props;
  Object.assign(handlesMap, handles);
});

var ELEMENT_DIFF_KEYS = ['nodeId', 'pageId', 'tagName', 'compName', 'id', 'className', 'style', 'src', 'mode', 'lazyLoad', 'showMenuByLongpress', 'useTemplate', 'isImage', 'isLeaf', 'isSimple', 'content', 'extra', 'animation'];
var TEXT_NODE_DIFF_KEYS = ['nodeId', 'pageId', 'content'];
var NEET_BEHAVIOR_NORMAL_CUSTOM_ELEMENT = ['movable-view', 'swiper-item', 'picker-view-column'];
var NEET_BEHAVIOR_NORMAL_CUSTOM_ELEMENT_PARENT = ['swiper', 'movable-area'];
var NOT_SUPPORT = ['IFRAME', 'A']; // The nodes that class and style need to be separated

var NEET_SPLIT_CLASS_STYLE_FROM_CUSTOM_ELEMENT = ['BUILTIN-COMPONENT', 'CUSTOM-COMPONENT'];
var USE_TEMPLATE = ['cover-image', 'movable-area', 'movable-view', 'swiper', 'swiper-item', 'icon', 'progress', 'rich-text', 'button', 'editor', 'form', 'INPUT', 'picker', 'slider', 'switch', 'TEXTAREA', 'navigator', 'camera', 'image', 'live-player', 'live-pusher', 'label', 'radio', 'radio-group', 'checkbox', 'checkbox-group', 'VIDEO', 'map', 'CANVAS', 'ad', 'official-account', 'open-data', 'web-view']; // The nodes that must be render as custom components

var NEET_RENDER_TO_CUSTOM_ELEMENT = [].concat(NOT_SUPPORT, NEET_SPLIT_CLASS_STYLE_FROM_CUSTOM_ELEMENT);
var IN_COVER = ['cover-view'];

function isEqual(a, b) {
  if (typeof a === 'number' && typeof b === 'number') {
    return parseInt(a * 1000, 10) === parseInt(b * 1000, 10);
  }

  return a === b;
}

function checkComponentAttr(instance, name, newData, extraClass) {
  if (extraClass === void 0) {
    extraClass = '';
  }

  var oldData = instance.data;
  var domNode = instance.domNode;
  var attrs = propsMap[name];
  newData.builtinComponentName = name;

  if (attrs && attrs.length) {
    for (var _iterator = _createForOfIteratorHelperLoose(attrs), _step; !(_step = _iterator()).done;) {
      var _step$value = _step.value,
          _name = _step$value.name,
          get = _step$value.get,
          _step$value$canBeUser = _step$value.canBeUserChanged,
          canBeUserChanged = _step$value$canBeUser === void 0 ? false : _step$value$canBeUser;
      var newValue = get(domNode);

      if (canBeUserChanged) {
        var oldValues = domNode.__oldValues;

        if (!oldData || !isEqual(newValue, oldData[_name]) || oldValues && !isEqual(newValue, oldValues[_name])) {
          newData[_name] = newValue;
          newData.forceUpdate = true;
        }
      } else if (!oldData || !isEqual(newValue, oldData[_name])) {
        newData[_name] = newValue;
      }
    }
  } // Add id/class/style/hidden/animation


  var newId = domNode.id;
  if (!oldData || oldData.id !== newId) newData.id = newId;
  var newClass = extraClass + " builtin-component-" + name + " node-" + domNode.$$nodeId + " " + (domNode.className || '');
  if (!oldData || oldData.className !== newClass) newData.className = newClass;
  var newStyle = domNode.style.cssText;
  if (!oldData || oldData.style !== newStyle) newData.style = newStyle;
  var newHidden = domNode.getAttribute('hidden') || false;
  if (!oldData || oldData.hidden !== newHidden) newData.hidden = newHidden;
  var newAnimation = domNode.getAttribute('animation');
  if (!oldData || oldData.animation !== newAnimation) newData.animation = newAnimation;
}

function filterNodes(domNode, level, component) {
  var childNodes = domNode.childNodes || [];
  if (!childNodes.map) return []; // Tags are not supported and child nodes are not rendered

  if (NOT_SUPPORT.indexOf(domNode.tagName) >= 0) return [];
  return childNodes.map(function (child) {
    var domInfo = child.$$domInfo;
    if (domInfo.type !== 'element' && domInfo.type !== 'text') return; // Add default class

    domInfo.className = domInfo.type === 'element' ? "h5-" + domInfo.tagName + " node-" + domInfo.nodeId + " " + (domInfo.className || '') : '';
    domInfo.domNode = child; // Special node

    if (NEET_SPLIT_CLASS_STYLE_FROM_CUSTOM_ELEMENT.indexOf(child.tagName) >= 0) {
      if (domInfo.tagName === 'builtin-component' && NEET_BEHAVIOR_NORMAL_CUSTOM_ELEMENT.indexOf(child.behavior) !== -1) {
        domInfo.compName = child.behavior;
        domInfo.extra = {
          hidden: child.getAttribute('hidden') || false
        }; // Add special component props

        var props = propsMap[child.behavior] || {};

        if (props && props.length) {
          props.forEach(function (_ref) {
            var name = _ref.name,
                get = _ref.get;
            domInfo.extra[name] = get(child);
          });
        }

        if (child.children.length && level > 0) {
          domInfo.childNodes = filterNodes(child, level - 1, component);
        }

        return domInfo;
      } // id and style are excluded


      domInfo.className = "h5-" + domInfo.tagName + " " + (domInfo.tagName === 'builtin-component' ? 'builtin-' + child.behavior : '');
      domInfo.id = '';
      domInfo.style = '';
    } // Check image node


    domInfo.isImage = domInfo.type === 'element' && domInfo.tagName === 'img';

    if (domInfo.isImage) {
      domInfo.src = child.src || '';
      domInfo.mode = child.getAttribute('mode') || '';
      domInfo.lazyLoad = !!child.getAttribute('lazy-load');
      domInfo.showMenuByLongpress = !!child.getAttribute('show-menu-by-longpress');
    } else {
      domInfo.src = '';
      domInfo.mode = '';
      domInfo.lazyLoad = false;
      domInfo.showMenuByLongpress = false;
    } // Check wheather use template


    var templateName = domInfo.tagName === 'builtin-component' ? child.behavior : child.tagName;
    domInfo.useTemplate = !domInfo.isImage && USE_TEMPLATE.indexOf(templateName) !== -1;

    if (domInfo.useTemplate) {
      var compName = componentNameMap[templateName.toLowerCase()];
      var extra = {};
      if (compName) checkComponentAttr(domInfo, compName, extra, "h5-" + domInfo.tagName + " " + (domInfo.tagName === 'builtin-component' ? 'builtin-' + child.behavior : ''));
      extra.pageId = domInfo.pageId;
      extra.nodeId = domInfo.nodeId;
      extra.inCover = component.data.inCover;
      extra.hasChildren = !!child.childNodes.length;
      domInfo.extra = extra;

      if (NEET_BEHAVIOR_NORMAL_CUSTOM_ELEMENT_PARENT.indexOf(templateName) !== -1) {
        var _childNodes = filterNodes(child, 0, component) || [];

        extra.childNodes = _childNodes.map(function (childNode) {
          var copyChildNode = Object.assign({}, childNode);
          delete copyChildNode.domNode;
          return copyChildNode;
        });
      }
    } // Check child nodes


    domInfo.isLeaf = !domInfo.isImage && domInfo.type === 'element' && !child.children.length && NEET_RENDER_TO_CUSTOM_ELEMENT.indexOf(child.tagName.toUpperCase()) === -1;

    if (domInfo.isLeaf) {
      domInfo.content = child.childNodes.map(function (childNode) {
        return childNode.$$domInfo.type === 'text' ? childNode.textContent : '';
      }).join('');
    } // Check simple node that can be rendered as view


    domInfo.isSimple = !domInfo.isImage && !domInfo.useTemplate && !domInfo.isLeaf && domInfo.type === 'element' && NEET_RENDER_TO_CUSTOM_ELEMENT.indexOf(child.tagName) === -1 && level > 0;

    if (domInfo.isSimple) {
      domInfo.content = '';
      domInfo.childNodes = filterNodes(child, level - 1, component);
    }

    return domInfo;
  }).filter(function (child) {
    return !!child;
  });
}

function checkDiffChildNodes(newChildNodes, oldChildNodes) {
  if (newChildNodes.length !== oldChildNodes.length) return true;

  for (var i = 0, len = newChildNodes.length; i < len; i++) {
    var newChild = newChildNodes[i];
    var oldChild = oldChildNodes[i];
    if (newChild.type !== oldChild.type) return true;
    var keys = newChild.type === 'element' ? ELEMENT_DIFF_KEYS : TEXT_NODE_DIFF_KEYS;

    for (var _iterator = _createForOfIteratorHelperLoose(keys), _step; !(_step = _iterator()).done;) {
      var key = _step.value;
      var newValue = newChild[key];
      var oldValue = oldChild[key];

      if (typeof newValue === 'object' && !Array.isArray(newValue)) {
        if (typeof oldValue !== 'object') return true;

        if (key === 'extra' && newValue.forceUpdate) {
          newValue.forceUpdate = false;
          return true;
        }

        var objectKeys = Object.keys(newValue);

        for (var _i = 0, _objectKeys = objectKeys; _i < _objectKeys.length; _i++) {
          var objectKey = _objectKeys[_i];
          if (!isEqual(newValue[objectKey], oldValue[objectKey])) return true;
        }
      } else if (!isEqual(newValue, oldValue)) {
        return true;
      }
    } // Diff children


    var newGrandChildNodes = newChild.childNodes || [];
    var oldGrandChildNodes = oldChild.childNodes || [];

    if (newGrandChildNodes.length || oldGrandChildNodes.length) {
      var checkRes = checkDiffChildNodes(newGrandChildNodes, oldGrandChildNodes);
      if (checkRes) return true;
    }
  }

  return false;
}

// Deal the nodes that needn't transfrom to custom component
function dealWithLeafAndSimple(childNodes, onChildNodesUpdate) {
  if (childNodes && childNodes.length) {
    childNodes = childNodes.map(function (originChildNode) {
      var childNode = Object.assign({}, originChildNode);

      if (childNode.isImage || childNode.isLeaf || childNode.isSimple || childNode.useTemplate) {
        childNode.domNode.$$clearEvent('$$childNodesUpdate');
        childNode.domNode.addEventListener('$$childNodesUpdate', onChildNodesUpdate);
      }

      delete childNode.domNode;
      childNode.childNodes = dealWithLeafAndSimple(childNode.childNodes, onChildNodesUpdate);
      return childNode;
    });
  }

  return childNodes;
}

function init (instance, data) {
  var domNode = instance.domNode;
  var tagName = domNode.tagName;
  if (USE_TEMPLATE.indexOf(tagName) !== -1 || USE_TEMPLATE.indexOf(domNode.behavior) !== -1) return;

  if (tagName === 'BUILTIN-COMPONENT') {
    // BuildIn component
    data.builtinComponentName = domNode.behavior;
    var builtinComponentName = componentNameMap[data.builtinComponentName];
    if (builtinComponentName) checkComponentAttr(instance, builtinComponentName, data);else console.warn("value \"" + data.builtinComponentName + "\" is not supported for builtin-component's behavior");
  } else if (tagName === 'CUSTOM-COMPONENT') {
    // Custom component
    data.customComponentName = domNode.behavior;
    data.nodeId = instance.nodeId;
    data.pageId = instance.pageId;
  } else if (NOT_SUPPORT.indexOf(tagName) >= 0) {
    // Not supported component
    data.builtinComponentName = 'not-support';
    data.content = domNode.textContent;
  } else {
    // Could be replaced html tag
    var _builtinComponentName = componentNameMap[tagName.toLowerCase()];
    if (_builtinComponentName) data.builtinComponentName = _builtinComponentName;
  }
}

// eslint-disable-next-line import/no-extraneous-dependencies
function getInitialProps () {
  {
    return {
      properties: {
        inCover: {
          type: Boolean,
          value: false
        }
      }
    };
  }
}

// eslint-disable-next-line import/no-extraneous-dependencies
function getId (instance) {
  var nodeId, pageId;

  {
    nodeId = instance.dataset.privateNodeId;
    pageId = instance.dataset.privatePageId;
  }

  return {
    nodeId: nodeId,
    pageId: pageId
  };
}

// eslint-disable-next-line import/no-extraneous-dependencies
function getLifeCycle (_ref) {
  var init = _ref.init,
      mount = _ref.mount,
      _ready = _ref.ready,
      unmount = _ref.unmount;

  {
    var attached = function attached() {
      return mount.apply(this, arguments);
    };

    var detached = function detached() {
      return unmount.apply(this, arguments);
    };

    return {
      lifetimes: {
        attached: attached,
        detached: detached
      },
      // Keep compatibility to wx base library version < 2.2.3
      attached: attached,
      detached: detached,
      created: function created() {
        init.apply(this, arguments);
      },
      ready: function ready() {
        _ready.apply(this, arguments);
      }
    };
  }
}

var _render$$$adapter$1 = render__default.$$adapter,
    cache$2 = _render$$$adapter$1.cache,
    tool = _render$$$adapter$1.tool; // The number of levels of dom subtrees rendered as custom components

var MAX_DOM_SUB_TREE_LEVEL = 10;
var DOM_SUB_TREE_LEVEL = 10;

var config = _extends({
  data: {
    builtinComponentName: '',
    // the builtIn component name
    customComponentName: '',
    // current render custom component name
    childNodes: []
  }
}, getInitialProps(), {
  options: {
    addGlobalClass: true // global style

  },
  methods: _extends({
    // Watch child nodes update
    onChildNodesUpdate: function onChildNodesUpdate() {
      // Node unomunted
      if (!this.pageId || !this.nodeId) return; // child nodes update

      var childNodes = filterNodes(this.domNode, DOM_SUB_TREE_LEVEL - 1, this);

      if (checkDiffChildNodes(childNodes, this.data.childNodes)) {
        this.setData({
          childNodes: dealWithLeafAndSimple(childNodes, this.onChildNodesUpdate)
        });
      } // dispatch child update


      var childNodeStack = [].concat(childNodes);
      var childNode = childNodeStack.pop();

      while (childNode) {
        if (childNode.type === 'element' && !childNode.isImage && !childNode.isLeaf && !childNode.isSimple && !childNode.useTemplate) {
          childNode.domNode.$$trigger('$$childNodesUpdate');
        }

        if (childNode.childNodes && childNode.childNodes.length) childNode.childNodes.forEach(function (subChildNode) {
          return childNodeStack.push(subChildNode);
        });
        childNode = childNodeStack.pop();
      }
    },
    // Watch node update
    onSelfNodeUpdate: function onSelfNodeUpdate() {
      // Node unomunted
      if (!this.pageId || !this.nodeId) return;
      var newData = {};
      var domNode = this.domNode;
      var data = this.data;
      var tagName = domNode.tagName;
      var newAttrData = newData;

      if (!this.__ready) {
        this.__readyData = newAttrData = {};
      }

      if (tagName === 'BUILTIN-COMPONENT') {
        // BuiltIn component
        if (data.builtinComponentName !== domNode.behavior) newData.builtinComponentName = domNode.behavior;
        var builtinComponentName = componentNameMap[domNode.behavior];
        if (builtinComponentName) checkComponentAttr(this, builtinComponentName, newAttrData);
      } else if (tagName === 'CUSTOM-COMPONENT') {
        // Custom component
        if (data.customComponentName !== domNode.behavior) newData.customComponentName = domNode.behavior;
        if (data.nodeId !== this.nodeId) data.nodeId = this.nodeId;
        if (data.pageId !== this.pageId) data.pageId = this.pageId;
      } else if (NOT_SUPPORT.indexOf(tagName) >= 0) {
        // Not support
        newData.builtinComponentName = 'not-support';
        if (data.content !== domNode.content) newData.content = domNode.textContent;
      } else {
        // Replaced html tag
        var _builtinComponentName = componentNameMap[tagName.toLowerCase()];
        if (_builtinComponentName) newData.builtinComponentName = _builtinComponentName;
      }

      this.setData(newData);
    },
    // Dom event
    onTouchStart: function onTouchStart(evt) {
      if (this.document && this.document.$$checkEvent(evt)) {
        callEvent('touchstart', evt, null, this.pageId);
      }
    },
    onTouchMove: function onTouchMove(evt) {
      if (this.document && this.document.$$checkEvent(evt)) {
        callEvent('touchmove', evt, null, this.pageId);
      }
    },
    onTouchEnd: function onTouchEnd(evt) {
      if (this.document && this.document.$$checkEvent(evt)) {
        callEvent('touchend', evt, null, this.pageId);
      }
    },
    onTouchCancel: function onTouchCancel(evt) {
      if (this.document && this.document.$$checkEvent(evt)) {
        callEvent('touchcancel', evt, null, this.pageId);
      }
    },
    onTap: function onTap(evt) {
      if (this.document && this.document.$$checkEvent(evt)) {
        callEvent('click', evt, {
          button: 0
        }, this.pageId); // 默认左键
      }
    },
    onImgLoad: function onImgLoad(evt) {
      var pageId = this.pageId;
      var originNodeId = evt.currentTarget.dataset.privateNodeId || this.nodeId;
      var originNode = cache$2.getNode(pageId, originNodeId);
      if (!originNode) return;
      callSimpleEvent('load', evt, originNode);
    },
    onImgError: function onImgError(evt) {
      var pageId = this.pageId;
      var originNodeId = evt.currentTarget.dataset.privateNodeId || this.nodeId;
      var originNode = cache$2.getNode(pageId, originNodeId);
      if (!originNode) return;
      callSimpleEvent('error', evt, originNode);
    },
    getDomNodeFromEvt: function getDomNodeFromEvt(eventName, evt) {
      if (!evt) return;
      var pageId = this.pageId;
      var originNodeId = this.nodeId;

      if (evt.currentTarget && evt.currentTarget.dataset.privateNodeId) {
        originNodeId = evt.currentTarget.dataset.privateNodeId;
      } else if (eventName && eventName.indexOf('canvas') === 0 && evt.target && evt.target.dataset.privateNodeId) {
        originNodeId = evt.target.dataset.privateNodeId;
      }

      return cache$2.getNode(pageId, originNodeId);
    }
  }, handlesMap)
});

var lifeCycles = getLifeCycle({
  init: function init() {
    var config = cache$2.getConfig(); // Resets global variables according to configuration

    if (config.optimization) {
      var domSubTreeLevel = +config.optimization.domSubTreeLevel;
      if (domSubTreeLevel >= 1 && domSubTreeLevel <= MAX_DOM_SUB_TREE_LEVEL) DOM_SUB_TREE_LEVEL = domSubTreeLevel;
    }
  },
  mount: function mount() {
    var _getId = getId(this),
        nodeId = _getId.nodeId,
        pageId = _getId.pageId;

    var data = {};
    this.nodeId = nodeId;
    this.pageId = pageId; // Record dom

    this.domNode = cache$2.getNode(pageId, nodeId);
    if (!this.domNode) return; // Store document

    this.document = cache$2.getDocument(pageId); // Listen global event

    this.onChildNodesUpdate = tool.throttle(this.onChildNodesUpdate.bind(this));
    this.domNode.$$clearEvent('$$childNodesUpdate');
    this.domNode.addEventListener('$$childNodesUpdate', this.onChildNodesUpdate);
    this.onSelfNodeUpdate = tool.throttle(this.onSelfNodeUpdate.bind(this));
    this.domNode.$$clearEvent('$$domNodeUpdate');
    this.domNode.addEventListener('$$domNodeUpdate', this.onSelfNodeUpdate); // init

    init(this, data);
    if (IN_COVER.indexOf(data.builtinComponentName) !== -1) this.data.inCover = true; // init child nodes

    var childNodes = filterNodes(this.domNode, DOM_SUB_TREE_LEVEL - 1, this);
    data.childNodes = dealWithLeafAndSimple(childNodes, this.onChildNodesUpdate);
    if (Object.keys(data).length) this.setData(data);
  },
  ready: function ready() {
    this.__ready = true;

    if (this.__readyData) {
      this.setData(this.__readyData);
      this.__readyData = null;
    }
  },
  unmount: function unmount() {
    this.nodeId = null;
    this.pageId = null;
    this.domNode = null;
    this.document = null;
  }
});
Component(_extends({}, config, {}, lifeCycles));
